import json

import requests
import PythonQt
import urllib3
import base64
import PhenarisLogger

BASE_URL = 'https://178.115.234.79:10443/api'  # TransporterModels URL
log = PhenarisLogger.PhenarisQueryLogger('PhenarisQuery.log')


class TransporterModelsWebService:
    def __init__(self, token):
        """
        Create an instance of TransporterModelsWebService

        :param token: Token to access API of Transporter Models
        """
        self.token = token
        self.http = urllib3.PoolManager(cert_reqs='CERT_NONE')

    def get_predictions(self):
        """
        Get all predictions of user

        :param appui:
        """
        url = BASE_URL
        # User-Agent maybe needed for ('Connection aborted.', BadStatusLine("''",))
        # 'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:86.0) Gecko/20100101 Firefox/86.0'
        headers = {'Authorization': 'Bearer ' + self.token.getToken()}
        response = self.http.request('GET', url + '/predictions', headers=headers)
        return response, response.status

    def get_prediction_with_id(self, id):
        """
        Returns prediction with id
        :param id:
        :return:
        """
        url = BASE_URL
        headers = {'Authorization': 'Bearer ' + self.token.getToken()}
        response = self.http.request('GET', url + '/predictions/' + str(id), headers=headers)
        return response, response.status

    def post_quickprediction(self, name, models, mol):
        """
        Performs a POST request to the Transporter Model API to make a quick prediction of the drawn molecule
        :param name: Name of the prediction
        :param models: List of models used for prediction
        :param mol: mol file of the drawn compound
        :return:
        """
        url = BASE_URL
        headers = {'Authorization': 'Bearer ' + self.token.getToken(),
                   'Content-Type': 'application/json; charset=utf-8'}
        mol_encoded = mol.encode('utf-8')
        mol_encoded_64 = base64.b64encode(mol_encoded)
        args = {'name': name,
                'models': models,
                'rawinput': mol_encoded_64}

        encoded_args = json.dumps(args)
        response = self.http.request('POST', url + '/quickprediction', body=encoded_args, headers=headers)
        return response, response.status

    def get_available_models(self):
        """
        Returns all available models for prediction
        :return:
        """
        url = BASE_URL
        headers = {'Authorization': 'Bearer ' + self.token.getToken()}
        response = self.http.request('GET', url + '/models', headers=headers)
        return response, response.status

    def post_upload_sdf(self, name, models, sdf):
        url = BASE_URL
        headers = {'Authorization': 'Bearer ' + self.token.getToken(),
                   'Content-Type': 'application/json; charset=utf-8'}

        sdf_encoded = sdf.encode('utf-8')
        sdf_encoded_64 = base64.b64encode(sdf_encoded)

        args = {
            'predictionname': name,
            'models': models,
            'description': 'Prediction of molecules from SDF-file',
            'rawinput': sdf_encoded_64
        }

        encoded_args = json.dumps(args)

        response = self.http.request('POST', url + '/predictions', body=encoded_args, headers=headers)
        return response.status

    def get_token(self):
        return self.token
